package org.eclipse.acceleo.mofqa.generation.templates.services;

import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.impl.DynamicEObjectImpl;
import org.eclipse.emf.ecore.util.EContentsEList;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.InstanceSpecification;
import org.eclipse.uml2.uml.Model;
import org.eclipse.uml2.uml.Operation;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Slot;
import org.eclipse.uml2.uml.Stereotype;

/**
 * @author Linda
 * UMLServices: methods to get UML elements/values.
 */
public class UMLServices {
	
	/**
	 * getSlot: obtain Slot element with the name of the specified property.
	 * @param instance InstanceSpecification where slot is set.
	 * @param property Property property to be found.
	 * @return Slot element
	 */
	public static Slot getSlot(InstanceSpecification instance, Property property) {
        for (Slot slot : instance.getSlots()) {
            String sPropName = slot.getDefiningFeature().getName();
            if (sPropName != null && sPropName.equals(property.getName())) {
                return slot;
            }
        }

        return null;
    }
	
	/**
	 * getTaggedValueClass: get a class referenced by a tagged value in a UML element
	 * @param el element referencing to class from a tagged value
	 * @param stereotype stereotype applied to element
	 * @param findProperty tagged value name
	 * @return referenced class
	 */
	public static Class getTaggedValueClass(Element el, Stereotype stereotype, String findProperty){
		DynamicEObjectImpl obj = (DynamicEObjectImpl) el.getValue(stereotype, findProperty);
		if (obj != null){
			return getClassFromDynamicObject(obj);
		}
		
		return null;
	}

	/**
	 * getTaggedValueClassAttributes: get properties referenced by a tagged value in a UML element
	 * @param el element with tagged value
	 * @param stereotype stereotype applied to element
	 * @param findProperty tagged value name
	 * @return List<Property> referenced attributes
	 */
	public static List<Property> getTaggedValueClassAttributes(Element el, Stereotype stereotype, String findProperty){
		DynamicEObjectImpl obj = (DynamicEObjectImpl) el.getValue(stereotype, findProperty);
		if (obj != null){
			return getPropertiesFromDynamicObject(obj);
		}
		
		return null;
	}

	/**
	 * getPropertiesFromDynamicObject: get property elements referenced by dynamic object
	 * @param objGen dynamic object referencing properties
	 * @return List<Property> referenced attributes
	 */
	public static List<Property> getPropertiesFromDynamicObject(Object objGen){
		DynamicEObjectImpl obj = (DynamicEObjectImpl) objGen;
		for (EContentsEList.FeatureIterator featureIterator = 
		        (EContentsEList.FeatureIterator)obj.eCrossReferences().iterator();
		       featureIterator.hasNext(); ){
			EObject eObject = (EObject)featureIterator.next();
			if (eObject instanceof Class){
				return ((Class) eObject).allAttributes();
			}
		}
		
		return null;
	}
	
	/**
	 * getClassFromDynamicObject: get class element referenced by dynamic object
	 * @param objGen dynamic object referencing class
	 * @return Class referenced class
	 */
	public static Class getClassFromDynamicObject(Object objGen){
		DynamicEObjectImpl obj = (DynamicEObjectImpl) objGen;
		for (EContentsEList.FeatureIterator featureIterator = 
		        (EContentsEList.FeatureIterator)obj.eCrossReferences().iterator();
		       featureIterator.hasNext(); ){
			EObject eObject = (EObject)featureIterator.next();
			if (eObject instanceof Class){
				return (Class) eObject;
			}
		}
		
		return null;
	}

	/**
	 * getClassesFromDynamicObject: get a list of classes referenced by dynamic object
	 * @param objGen dynamic object referencing classes
	 * @return List<Class> referenced class
	 */
	public static List<Class> getClassesFromDynamicObject(Object objGen){
		DynamicEObjectImpl obj = (DynamicEObjectImpl) objGen;
		for (EContentsEList.FeatureIterator featureIterator = 
		        (EContentsEList.FeatureIterator)obj.eCrossReferences().iterator();
		       featureIterator.hasNext(); ){
			EObject eObject = (EObject)featureIterator.next();
			if (eObject instanceof List<?>){
				return (List<Class>) eObject;
			}
		}
		
		return null;
	}

	/**
	 * getPackageFromDynamicObject: get a uml element referenced by dynamic object
	 * @param objGen dynamic object referencing element
	 * @return Element referenced element
	 */
	public static Element getPackageFromDynamicObject(Object objGen){
		DynamicEObjectImpl obj = (DynamicEObjectImpl) objGen;
		for (EContentsEList.FeatureIterator featureIterator = 
		        (EContentsEList.FeatureIterator)obj.eCrossReferences().iterator();
		       featureIterator.hasNext(); ){
			EObject eObject = (EObject)featureIterator.next();
			if (eObject instanceof Element){
				return (Element) eObject;
			}
		}
		
		return null;
	}

	/**
	 * getInstanceFromDynamicObject: get an instance referenced by dynamic object
	 * @param objGen dynamic object referencing instances
	 * @return InstanceSpecification referenced instance
	 */
	public static InstanceSpecification getInstanceFromDynamicObject(Object objGen){
		DynamicEObjectImpl obj = (DynamicEObjectImpl) objGen;
		for (EContentsEList.FeatureIterator featureIterator = 
		        (EContentsEList.FeatureIterator)obj.eCrossReferences().iterator();
		       featureIterator.hasNext(); ){
			EObject eObject = (EObject)featureIterator.next();
			if (eObject instanceof InstanceSpecification){
				return (InstanceSpecification) eObject;
			}
		}
		
		return null;
	}

	/**
	 * getOperationFromDynamicObject: get an operation referenced by dynamic object
	 * @param objGen dynamic object referencing operation
	 * @return Operation referenced operation
	 */
	public static Operation getOperationFromDynamicObject(Object objGen){
		DynamicEObjectImpl obj = (DynamicEObjectImpl) objGen;
		for (EContentsEList.FeatureIterator featureIterator = 
		        (EContentsEList.FeatureIterator)obj.eCrossReferences().iterator();
		       featureIterator.hasNext(); ){
			EObject eObject = (EObject)featureIterator.next();
			if (eObject instanceof Operation){
				return (Operation) eObject;
			}
		}
		
		return null;
	}

	/**
	 * getClassifier: get the class the instance was obtained from
	 * @param instance class instance
	 * @return Classifier instance classifier
	 */
	public static Classifier getClassifier(InstanceSpecification instance){
		Classifier clazz = instance.getClassifiers().get(0);
		return clazz;
	}
	
	public static String getElementNamespace(Classifier clazz){
		String namespace = "";
		for(Package parent : clazz.allOwningPackages()){
			if(!(parent instanceof Model)){
				namespace = parent.getName() + "." + namespace;
			}
		}
		return namespace;
	}
	
	public static String getElementNamespace(Package el){
		String namespace = el.getName();
		for(Package parent : el.allOwningPackages()){
			if(!(parent instanceof Model))
				namespace = parent.getName() + "." + namespace;
		}
		return namespace;
	}
	
}
