package org.eclipse.acceleo.mofqa.generation.templates.services;

import org.eclipse.uml2.uml.InstanceSpecification;
import org.eclipse.uml2.uml.Operation;
import org.eclipse.uml2.uml.Parameter;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Slot;
import org.eclipse.uml2.uml.Stereotype;
import org.eclipse.uml2.uml.Type;
import org.eclipse.uml2.uml.ValueSpecification;

import org.eclipse.emf.ecore.impl.DynamicEObjectImpl;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Enumeration;
import org.eclipse.uml2.uml.EnumerationLiteral;
import org.eclipse.uml2.uml.Package;

import java.util.Iterator;
import java.util.List;
import java.util.Random;

/**
 * @author Linda
 * UMLToJavaMappingServices methods for UML to Java mapping.
 */
public class UMLToJavaMappingServices {
	private static final Integer ALERT_DEFAULT_TIMEOUT = 5;				// timeout by default, in seconds
	private static final Integer PAGE_NAVIGATION_DEFAULT_TIMEOUT = 10;	// timeout by default, in seconds
	
	private static String getEnumerationClassName(Type pType){
		for(Package p : pType.allOwningPackages()){
			if(UMLVerificationServices.hasStereotype(p, "SUT")){
				String firstLett = p.getName().substring(0, 1).toUpperCase();
				String nextLett = p.getName().substring(1);
				return "Enumeration" + firstLett + nextLett;
			}
		}
		
		return "";
	}
	
	/**
	 * getUMLToJavaMappedType (from Property): from the uml datatype read, get the Java mapping (multiplicity is verified).
	 * @param umlProperty UML Property
	 * @return  string that represents the mapped datatype, if multiplicity > 1, the mapping results to a list.
	 */
	public static String getUMLToJavaMappedType(Property umlProperty){
		Type pType = umlProperty.getType();
		String umlTypeName = pType.getName();
		
		if (pType instanceof Enumeration){
			umlTypeName = getEnumerationClassName(pType) + "." + umlTypeName;
		}
		
		boolean isList = umlProperty.isMultivalued();
		return getTypeName(isList, umlTypeName);
	}
	
	/**
	 * getUMLToJavaMappedType (from Parameter): from the uml datatype read, get the Java mapping (multiplicity is verified).
	 * @param umlParam UML Parameter
	 * @return string that represents the mapped datatype, if multiplicity > 1, the mapping results to a list.
	 */
	public static String getUMLToJavaMappedType(Parameter umlParam){
		boolean isList = umlParam.isMultivalued();
		Type pType = umlParam.getType();
		String umlTypeName = pType.getName();
		if (pType instanceof Enumeration){
			umlTypeName = getEnumerationClassName(pType) + "." + umlTypeName;
		}
		return getTypeName(isList, umlTypeName);
	}
	
	/**
	 * getUMLToJavaMappedType (from Operation): from the uml datatype returned by uml operation, get the Java mapping (multiplicity is verified).
	 * @param umlOp UML Operation
	 * @return string that represents the mapped datatype, if multiplicity > 1, the mapping results to a list.
	 */
	public static String getUMLToJavaMappedType(Operation umlOp){
		int lower = umlOp.getLower();
		int upper = umlOp.getUpper();
		boolean isList = (lower > 1 || upper == -1 || upper > 1);
		Type pType = umlOp.getType();
		String umlTypeName = pType.getName();
		if (pType instanceof Enumeration){
			umlTypeName = getEnumerationClassName(pType) + "." + umlTypeName;
		}
		return getTypeName(isList, umlTypeName);
	}
	

	/**
	 * getTypeName get a string that represents the Java type from a uml type string.
	 * @param isList boolean value that says if type can have multiple (true) values or not (false).
	 * @param umlTypeName string that specifies uml datatype
	 * @return string that represents the mapped datatype, if multiplicity > 1, the mapping results to a list.
	 */
	private static String getTypeName(boolean isList, String umlTypeName){
		String javaTypeName = (isList) ? "List<" : "";
		
		if(umlTypeName == null)
			return "void";
		
		switch(umlTypeName){
			case "date":
				javaTypeName += "Date";
				break;
			case "int":
				javaTypeName += "Integer";
				break;
			case "":
			case "null":
				javaTypeName += "void";
				break;
			default:
				String firstLett = umlTypeName.substring(0, 1).toUpperCase();
				String nextLett = umlTypeName.substring(1);
				javaTypeName +=firstLett+nextLett;
				break;
		}
		
		if(isList)
			javaTypeName += ">";
		
		return javaTypeName;
	}
	
	/**
	 * getNullStringIfEmpty: verifies if string is empty and returns an equivalent string.
	 * @param verifyString string to verify
	 * @return if string is empty, returns the word "null", if not, returns the string into double quotes.
	 */
	public String getNullStringIfEmpty(String verifyString){
		if (verifyString == null)
			return "null";
		
		if (verifyString.isEmpty())
			return "null";
		
		return '"' + verifyString + '"';
	}
	
	/**
	 * @param prefix for name
	 * @return complete random name
	 */
	public static String getRandVarName(String prefix){
		Random rand = new Random();
		Integer randval = rand.nextInt(1000);
		return prefix+randval;
	}
	
	/**
	 * createValueObjectForPageComponent: get a string that instantiates a value object and asserts values
	 * @param dataObject an instancespecification, it has the values to assign
	 * @param dataPartObject a class (data partition) with the values to assign
	 * @param vObject a class that represents the value object, it has the attributes to be set
	 * @return  string for the VO instantiation and validation.
	 */
	public static String createValueObjectForPageComponent(Object dataObject, Object dataPartObject, Object vObject){
		if(vObject != null){
			Class cVObject = UMLServices.getClassFromDynamicObject((DynamicEObjectImpl) vObject);
			if(cVObject != null){
				List<Property> propsAVerVO = cVObject.getAllAttributes();
				String valuesList = "";
				
				if (dataObject != null && dataObject instanceof DynamicEObjectImpl){
					InstanceSpecification insDataObject = UMLServices.getInstanceFromDynamicObject((DynamicEObjectImpl) dataObject);
					// data specified as an instance
					if(insDataObject != null){
						//String assertions = "";
						for(Property p : propsAVerVO){
							ValueSpecification value = UMLServices.getSlot(insDataObject,p).getValues().get(0);	// value for property in instance
							if(value!=null){
								String val = '"' + value.stringValue() + '"';
								valuesList += (valuesList.isEmpty()) ?  val : "," + val;
							}
						}
					}
				}
						
				
				// data specified as partition
				if (dataPartObject != null && dataPartObject instanceof DynamicEObjectImpl){
					Class clazzDataPartition = UMLServices.getClassFromDynamicObject((DynamicEObjectImpl) dataPartObject);
					if(clazzDataPartition != null){
						Stereotype partSt = clazzDataPartition.getAppliedStereotype("AcceptanceCriteria::DataProvider::dataPartition");
						Class origClass = UMLServices.getClassFromDynamicObject(clazzDataPartition.getValue(partSt, "class"));
						for(Property p : propsAVerVO){
							//ValueSpecification value = UMLServices.getSlot(insDataObject,p).getValues().get(0);	// value for property in instance
							Property partProp = origClass.getAttribute(p.getName(),null);
							if(partProp != null){
								boolean isEnum = partProp.getType().eClass().getName().equals("Enumeration");
								boolean isList = partProp.isMultivalued();
								String typeName =UMLToJavaMappingServices.getUMLToJavaMappedType(partProp);
								String addQuotes = (UMLVerificationServices.isStringType(partProp)) ? "\"" : "";
			
								ValueSpecification value = p.getDefaultValue();
								if(value == null){
									//generate value
									String val = generateDummyValueForProperty(typeName);
									valuesList += (valuesList.isEmpty()) ? addQuotes + val + addQuotes : "," + addQuotes + val + addQuotes;
								}else{
									String val = value.stringValue();
									if(isEnum) val = typeName + "." + val;  
									val = (UMLVerificationServices.isStringType(p)) ? "\"" + val + "\"": val;
									if(isList) val = "Arrays.asList(" + val + ")";
									valuesList += (valuesList.isEmpty()) ? addQuotes + val + addQuotes : "," + addQuotes + val + addQuotes;
								}
							}
						}
					}
				}
			
				String voType = cVObject.getName().substring(0, 1).toUpperCase() + cVObject.getName().substring(1)+"VO";
				return "new "+ voType +"("+ valuesList +")";
			}
		}
		
		
		return "null";
	}
	
	/**
	 * createNewStatesForPageComponent: get a string that creates a list of new states caused with component interaction
	 * @param dataObjectList a list of classes with new state information
	 * @param listName the name for the states list (variable name)
	 * @return  string for the list instantiation and validation.
	 */
	public static String createNewStatesForPageComponent(Object dataObjectList, String listName){
		String listString = "// New states generated with component interaction.\n";
		listString += "\t\tList<PageComponent> "+listName+" = new ArrayList<PageComponent>();\n";
		
		if (dataObjectList != null && dataObjectList instanceof List){
			listString += "";
			for(Object newStateObj : (List<Object>) dataObjectList){
				if (newStateObj != null && newStateObj instanceof DynamicEObjectImpl){
					Class newState = UMLServices.getClassFromDynamicObject((DynamicEObjectImpl) newStateObj);				
					if(newState != null){
						Stereotype pageCompStereotype = newState.getAppliedStereotype("AcceptanceCriteria::ContentSpecification::pageComponent");
						Object idComp = newState.getValue(pageCompStereotype, "id");
						Object hiddenComp = newState.getValue(pageCompStereotype, "hidden");
						Object enabledComp = newState.getValue(pageCompStereotype, "enabled");
						Object cssComp = newState.getValue(pageCompStereotype, "cssClass");
						Object alttextComp = newState.getValue(pageCompStereotype, "altText");
						Object typeComp = newState.getValue(pageCompStereotype, "type");
												
						Object voComp = newState.getValue(pageCompStereotype, "valueObject");
						Object dataComp = newState.getValue(pageCompStereotype, "data");
						Object partComp = newState.getValue(pageCompStereotype, "partition");
						
						listString += "\t\tauxPC = new PageComponent("
								+ ((idComp == null) ? "null" : "\"" + idComp.toString() + "\"") + ", "
								+ ((hiddenComp == null) ? "false" : hiddenComp.toString()) + ", "
								+ ((enabledComp == null) ? "true" : enabledComp.toString()) + ", "
								+ ((cssComp == null) ? "null" : "\"" + cssComp.toString() + "\"") + ", "
								+ "false, "
								+ ((alttextComp == null) ? "null" : "\"" + alttextComp.toString() + "\"") + ", "
								+ createValueObjectForPageComponent(dataComp,partComp,voComp) + ", "
								+ "null, "
								+ "null, "
								+ ((typeComp == null) ? "PageComponentType.TEXT" : "PageComponentType."+((EnumerationLiteral)typeComp).getName().toUpperCase()) + ", "
								+ "new ArrayList<PageComponent>());\n";
						listString += "\t\t"+listName+".add(auxPC);\n";
					}
				}
			}
		}
		
		return listString;
	}
	
	
	/**
	 * createAlertMsgForPageComponent: generate a string with sentence to instantiate an AlertMsg element.
	 * @param alertObject DynamicEObjectImpl read from model with alert data
	 * @return string with instantiation
	 */
	public static String createAlertMsgForPageComponent(Object alertObject){
		if (alertObject != null && alertObject instanceof DynamicEObjectImpl){
			Class cAlertObject = UMLServices.getClassFromDynamicObject((DynamicEObjectImpl) alertObject);
			if(cAlertObject != null){
				Stereotype appliedSt = cAlertObject.getAppliedStereotype("AcceptanceCriteria::ContentSpecification::pageMessage");
				Object textObj = cAlertObject.getValue(appliedSt,"text");
				if(textObj == null)
					return "null";
				
				Object timeoutObj = cAlertObject.getValue(appliedSt,"timeout");
				String timeoutVal =  (timeoutObj == null) ? ALERT_DEFAULT_TIMEOUT.toString() : timeoutObj.toString();
				
				return "new AlertMsg("+ '"' + textObj.toString() + '"' +","+ timeoutVal +")";
			}	
		
		}
		
		return "null";
	}


	/**
	 * createPageNavigationForPageComponent: generates a string that instantiates a page element.
	 * @param navObject DynamicEObjectImpl read from UML model that represents the page element to instantiate.
	 * @param navName page variable name
	 * @return string with instantiation
	 */
	public static String createPageNavigationForPageComponent(Object navObject, String navName){
		String pageInstance = "Page "+navName+" = null;";
		
		if (navObject != null && navObject instanceof DynamicEObjectImpl){
			Element cNavObject = (Element)UMLServices.getPackageFromDynamicObject((DynamicEObjectImpl) navObject);
			if(cNavObject != null){
				Stereotype appliedSt = cNavObject.getAppliedStereotype("AcceptanceCriteria::ContentSpecification::page");
				Object urlObj = cNavObject.getValue(appliedSt,"url");
				Object titleObj = cNavObject.getValue(appliedSt,"title");
				Object timeoutObj = cNavObject.getValue(appliedSt,"timeout");
				
				if(urlObj != null || titleObj != null || timeoutObj != null){
					String urlVal = (urlObj == null) ? "null" : '"' + urlObj.toString()+ '"';
					String titleVal =  (titleObj == null) ? "null" : '"' + titleObj.toString()+ '"';
					String timeoutVal =  (timeoutObj == null) ? PAGE_NAVIGATION_DEFAULT_TIMEOUT.toString() : timeoutObj.toString();
					
					Object alertObj = cNavObject.getValue(appliedSt, "triggerMsg");
					String alertMsg = "null";
					if(alertObj!= null){
						Class alertClass = UMLServices.getClassFromDynamicObject(alertObj);
						if(alertClass != null){
							Stereotype st = alertClass.getAppliedStereotype("AcceptanceCriteria::ContentSpecification::pageMessage");
							Object textMsg = alertClass.getValue(st,"text");
							Object timeoutMsg = alertClass.getValue(st,"timeout");
							alertMsg = "new AlertMsg(" + ((textMsg == null) ? "null" : "\"" + textMsg.toString() + "\"") 
										+ ", " + ((timeoutMsg == null) ? ALERT_DEFAULT_TIMEOUT.toString() : timeoutMsg.toString())
										+ ")";
						}
					}
					
					//Object navPageComponents = cNavObject.getValue(appliedSt, "pageComponents");
					String navInstantiation = "";
					//if(navPageComponents != null && navPageComponents instanceof List){
						//for(Object navComp : (List<Object>) navPageComponents){
					for(Element comp : ((Package)cNavObject).allOwnedElements()){
						if(comp instanceof Class && UMLVerificationServices.hasStereotype(comp, "pageComponent")){
							Class compClass = (Class)comp;
							if(compClass != null){
								Stereotype st = compClass.getAppliedStereotype("AcceptanceCriteria::ContentSpecification::pageComponent");
								Object origComp = compClass.getValue(st,"originalState");
								
								if((Boolean)origComp || origComp == null){
									Object idComp = compClass.getValue(st,"id");
									Object hiddenComp = compClass.getValue(st,"hidden");
									Object enabledComp = compClass.getValue(st,"enabled");
									Object cssComp = compClass.getValue(st,"cssClass");
									Object alttextComp = compClass.getValue(st,"altText");
									Object voComp = compClass.getValue(st,"valueObject");
									Object dataComp = compClass.getValue(st,"data");
									Object partComp = compClass.getValue(st,"partition");
									Object alertComp = compClass.getValue(st,"alert");
									Object navToComp = compClass.getValue(st,"navigateTo");
									Object typeComp = compClass.getValue(st,"type");
									Object newstatesComp = compClass.getValue(st,"newStates");
									
									String pageNavName = getRandVarName(navName + "Nav");
									String newStNavName = getRandVarName(navName + "NewStates");
									
									navInstantiation += createPageNavigationForPageComponent(navToComp, pageNavName);
									navInstantiation += createNewStatesForPageComponent((List<Object>) newstatesComp,newStNavName);
									navInstantiation += "\n\t\t"+ navName + "Comps.add(new PageComponent("+
											((idComp==null) ? "null" : "\"" + idComp.toString() + "\"") + ", " +
											((hiddenComp==null) ? "false" : hiddenComp.toString()) + ", " +
											((enabledComp==null) ? "true" : enabledComp.toString()) + ", " +
											((cssComp==null) ? "null" : "\"" + cssComp.toString() + "\"") + ", " +
											true + ", " +
											((alttextComp==null) ? "null" : "\"" + alttextComp.toString() + "\"") + ", " +
											createValueObjectForPageComponent(dataComp,partComp,voComp) + ", " +
											createAlertMsgForPageComponent(alertComp) + ", " +
											pageNavName + ", " +
											"PageComponentType." + ((EnumerationLiteral)typeComp).getName().toUpperCase() + ", " +
											newStNavName + "));\n";
								}
							}
						}
					}
					
					pageInstance = "\t\tPage "+navName+" = new Page("+ urlVal +","+ titleVal +","+ timeoutVal +"," + alertMsg + ", null);\n";
					pageInstance += "\t\tList<PageComponent> " + navName + "Comps = new ArrayList<PageComponent>();\n";
					pageInstance += navInstantiation;
					pageInstance += "\t\t" + navName + ".setPageComponents(" + navName + "Comps);\n";		
				}
			}	
		}
		
		return pageInstance;
	}
	
	
	/**
	 * createClassInstantiation: get a string that instantiates a class assigning required attributes and defining setters for non-required attributes
	 * @param clazz the classifier
	 * @param instance instance with values to assign
	 * @param objName name for object
	 * @return  string for the instantiation
	 */
	public static String createClassInstantiation(Classifier clazz, InstanceSpecification instance, String objName){
		if(clazz != null && instance != null){
			List<Property> propsClass = clazz.getAllAttributes();
			String valuesList = "";
			String settersList = "\t\t";
			for(Property p : propsClass){
				Slot slot = UMLServices.getSlot(instance,p);
				
				boolean isList = p.isMultivalued();
				boolean isEnum = p.getType().eClass().getName().equals("Enumeration");
				
				if(slot != null){
					ValueSpecification value = slot.getValues().get(0);	// value for property in instance
					String typeName = UMLToJavaMappingServices.getUMLToJavaMappedType(p);
					// propiedades requeridas al constructor
					if(UMLVerificationServices.isRequiredAttribute(p)){
						if(value!=null){
							String val = value.stringValue();
							if(isEnum) val = typeName + "." + val; 
							val = (UMLVerificationServices.isStringType(p)) ? "\"" + val + "\"": val;
							if(isList) val = "Arrays.asList(" + val + ")";
							valuesList += (valuesList.isEmpty()) ? val : "," + val;
						}
					}
					// propiedades no requeridas a setters
					else{
						String firstLett = p.getName().substring(0, 1).toUpperCase();
						String nextLett = p.getName().substring(1);
						if(value!=null){
							String val = value.stringValue();
							if(isEnum) val = typeName + "." + val;  
							val = (UMLVerificationServices.isStringType(p)) ? "\"" + val + "\"": val;
							if(isList) val = "Arrays.asList(" + val + ")";
							settersList += objName+".set"+firstLett+nextLett+"("+ val +");\n\t\t";	
						}
					}
				}
			}
				
			String firstLett = clazz.getName().substring(0, 1).toUpperCase();
			String nextLett = clazz.getName().substring(1);
			
			return firstLett+nextLett+" "+objName+" = new "+firstLett+nextLett+"("+valuesList+");\n"+settersList;
		}
		
		
		return null;
	}
	
	
	/**
	 * createDummyInstancesForPartitions: for classes that represent data partitions, generate the instantiation string. 
	 * Data not specified in partitions are randomly generated.
	 * @param clazz the classifier
	 * @param objName name for object
	 * @return  string for the instantiation
	 */
	public static String createDummyInstancesForPartitions(Class clazz, String objName){
		if(clazz != null){
			Stereotype partStereotype = clazz.getAppliedStereotype("AcceptanceCriteria::DataProvider::dataPartition");
			Class classForPartition = UMLServices.getClassFromDynamicObject(clazz.getValue(partStereotype, "class"));	// the original class
			String valuesList = "";
			String settersList = "\t\t";
			for(Property p : classForPartition.getAllAttributes()){
				// find attribute in partition class
				Property partProp = clazz.getAttribute(p.getName(), p.getType());
				boolean isEnum = p.getType().eClass().getName().equals("Enumeration");
				boolean isList = p.isMultivalued();
				String typeName =UMLToJavaMappingServices.getUMLToJavaMappedType(p);
				String addQuotes = (UMLVerificationServices.isStringType(p)) ? "\"" : "";
				// propiedades requeridas al constructor
				if(UMLVerificationServices.isRequiredAttribute(p)){
					if(partProp == null){
						//generate value
						String val = generateDummyValueForProperty(typeName);
						valuesList += (valuesList.isEmpty()) ? addQuotes + val + addQuotes : "," + addQuotes + val + addQuotes;
					}else{
						ValueSpecification value = partProp.getDefaultValue();
						if(value!=null){
							String val = value.stringValue();
							if(isEnum) val = typeName + "." + val;  
							val = (UMLVerificationServices.isStringType(p)) ? "\"" + val + "\"": val;
							if(isList) val = "Arrays.asList(" + val + ")";
							valuesList += (valuesList.isEmpty()) ? addQuotes + val + addQuotes : "," + addQuotes + val + addQuotes;
						}
					}
				}
				// propiedades no requeridas a setters
				else{
					if(partProp != null){
						String firstLett = p.getName().substring(0, 1).toUpperCase();
						String nextLett = p.getName().substring(1);
						ValueSpecification value = partProp.getDefaultValue();
						if(value!=null){
							String val = value.stringValue();
							if(isEnum) val = typeName + "." + val;
							val = (UMLVerificationServices.isStringType(p)) ? "\"" + val + "\"": val;
							if(isList) val = "Arrays.asList(" + val + ")";
							settersList += objName+".set"+firstLett+nextLett+"("+ addQuotes + val + addQuotes +");\n\t\t";	
						}
					}
				}
			}
			
			String firstLett = classForPartition.getName().substring(0, 1).toUpperCase();
			String nextLett = classForPartition.getName().substring(1);
			return firstLett+nextLett+" "+objName+" = new "+firstLett+nextLett+"("+valuesList+");\n"+settersList;
		}
		
		return null;
	}
	
	
	/**
	 * generateDummyValueForProperty: generate a random value of type specified.
	 * @param pTypeName type of value to generate
	 * @return string with the generated value
	 */
	public static String generateDummyValueForProperty(String pTypeName){
		Random rand = new Random();
		switch(pTypeName){
			case "int":
			case "Integer":
				Integer number = rand.nextInt();
				return number.toString();
				
			case "String":
				return "Dummy Text";
				
			case "boolean":
				Boolean val = rand.nextBoolean();
				return val.toString();
		}
		
		return null;
	}
	
	/**
	 * createAssertionForPartitionVerification: partition verification consists of comparing the values of the properties 
	 * specified in partition, set for the instance and the default value set in the partition.
	 * @param instanceName
	 * @param postpartition
	 * @return string with assert sentences (1 assertion per attribute specified in partition)
	 */
	public static String createAssertionForPartitionVerification(String instanceName, Class postpartition){
		String assertions = "";
		for(Property p:postpartition.getAllAttributes()){
			ValueSpecification value = p.getDefaultValue();
			if(value != null){
				String val = value.stringValue();
				String typeName = UMLToJavaMappingServices.getUMLToJavaMappedType(p);
				String addQuotes = (UMLVerificationServices.isStringType(p)) ? "\"" : "";
				boolean isEnum = p.getType().eClass().getName().equals("Enumeration");
				boolean isBool = p.getType().getName().equals("Boolean") || p.getType().getName().equals("boolean");
				boolean isList = p.isMultivalued();
				if(isEnum) val = typeName + "." + val;  
				val = (UMLVerificationServices.isStringType(p)) ? "\"" + val + "\"": val;
				if(isBool) val = "new Boolean("+ val +")";
				if(isList) val = "Arrays.asList(" + val + ")";
				String firstLett = p.getName().substring(0, 1).toUpperCase();
				String nextLett = p.getName().substring(1);
				assertions += "\n\t\tAssert.assertEquals("+instanceName+".get"+firstLett+nextLett+"(), "+ addQuotes + val + addQuotes +");";
			}
		}
		
		return assertions;
	}
	
	/**
	 * assertResultsForConditions: generate java sentences that instantiate example objects, execute tested operations,
	 * verify the result and assert final values for instances.
	 * @param clazz <<condition>> class
	 * @param testOp uml operation (for role object, previously defined) to test
	 * @return string with java sentences that create data elements, operation calls, result and postcondition validations
	 */
	public static String assertResultsForConditions(Class clazz, Operation testOp, String roleName){
		Random rand = new Random();
		Integer randval = rand.nextInt(1000);
		
		// parts of the string to return
		String opCall = getUMLToJavaMappedType(testOp) + " result"+randval+" = "+ roleName +"." + testOp.getName();	// line where operation is called
		String parameters = "";	// parameters for operation
		String constructors = "";	// set of instantiations before operation call
		String assertion = "";	// string with verification of result and postconditions
		
		Stereotype condStereotype = clazz.getAppliedStereotype("AcceptanceCriteria::ConstraintSpecification::condition");
		String prefixDummy = clazz.getName().toLowerCase();	// the prefix used for the instances will be the condition class name
		String prefixPostDummy = "post"+clazz.getName();	// the prefix used for the instances of postConditions
		Integer counter = 0;
		
		// the result to assert
		String result = clazz.getValue(condStereotype,"result").toString();
		assertion = "System.out.println(\"Test: function return verification.\");";
		assertion += "\n\t\tAssert.assertEquals(String.valueOf(result"+randval+"), \""+result+"\");";
		
		// parameters for method to test are generated from the specified partitions
		Object belongsToPartitions = clazz.getValue(condStereotype, "belongsToPartition");
		if(belongsToPartitions != null){
			// verify states of objects by partition
			Object willBelongToPartitions = clazz.getValue(condStereotype,"willBelongToPartition");
			if(belongsToPartitions instanceof List){
				Iterator<?> postfeatureIterator = ((List<?>) willBelongToPartitions).iterator();
				for (Iterator<?> featureIterator = ((List<?>) belongsToPartitions).iterator(); featureIterator.hasNext(); ){
					Class partition = UMLServices.getClassFromDynamicObject(featureIterator.next());
					String instanceName = prefixDummy+(counter).toString();
					
					constructors += createDummyInstancesForPartitions(partition, instanceName);
					parameters += (parameters=="") ? instanceName : ", "+instanceName;
					
					Class postpartition = UMLServices.getClassFromDynamicObject(postfeatureIterator.next());
					String postinstanceName = prefixPostDummy+(counter).toString();
					constructors += createDummyInstancesForPartitions(postpartition, postinstanceName);
					
					assertion += createAssertionForPartitionVerification(instanceName, postpartition);
					
					counter++;
				}
			}
		}
		
		
		// parameters for method to test are defined by data elements
		Object preconditions = clazz.getValue(condStereotype,"given");
		if(preconditions != null){
			// compare states of previous objects with these new objects
			Object postconditions = clazz.getValue(condStereotype,"then");

			if(preconditions instanceof List){
				Iterator<?> postfeatureIterator = ((List<?>) postconditions).iterator();
				for (Iterator<?> prefeatureIterator = ((List<?>) preconditions).iterator(); prefeatureIterator.hasNext(); ){
					InstanceSpecification partition = UMLServices.getInstanceFromDynamicObject(prefeatureIterator.next());
					String instanceName = prefixDummy+(counter).toString();
					constructors += createClassInstantiation(UMLServices.getClassifier(partition), partition, instanceName);
					parameters += (parameters=="") ? instanceName : ", "+instanceName;
					
					InstanceSpecification postpartition = UMLServices.getInstanceFromDynamicObject(postfeatureIterator.next());
					String postinstanceName = prefixPostDummy+(counter).toString();
					constructors += createClassInstantiation(UMLServices.getClassifier(postpartition), postpartition, postinstanceName);
					assertion += "\n\t\tAssert.assertEquals("+instanceName+", "+postinstanceName+");\t//postcondition validation\n";
					
					counter++;
				}
			}
		}
		
		return constructors + "\n\t\t" + opCall + "("+parameters+");\n\t\t" + assertion;
	}
}
